print("import: 'jedi'")
import jedi

print("import: 'jedi.api'")
import jedi.api

print("import: 'jedi.common'")
import jedi.common

print("import: 'jedi.inference'")
import jedi.inference

print("import: 'jedi.inference.compiled'")
import jedi.inference.compiled

print("import: 'jedi.inference.compiled.subprocess'")
import jedi.inference.compiled.subprocess

print("import: 'jedi.inference.gradual'")
import jedi.inference.gradual

print("import: 'jedi.inference.value'")
import jedi.inference.value

print("import: 'jedi.plugins'")
import jedi.plugins

